// $Id: GPX.Format.js,v 1.1.2.4 2008/08/29 18:54:00 openwereld Exp $

/* Copyright (c) 2006-2007 MetaCarta, Inc., published under a modified BSD license.
 * See http://svn.openlayers.org/trunk/openlayers/repository-license.txt 
 * for the full text of the license. */

/**
 * @requires OpenLayers/Format/XML.js
 * @requires OpenLayers/Feature/Vector.js
 * @requires OpenLayers/Geometry/Point.js
 * @requires OpenLayers/Geometry/LineString.js
 *
 * Class: OpenLayers.Format.GPX
 * Read/write GPX parser. Create a new instance with the 
 *     <OpenLayers.Format.GPX> constructor.
 *
 * Inherits from:
 *  - <OpenLayers.Format.XML>
 */
MappingKit.Format.GPX = OpenLayers.Class(OpenLayers.Format.XML, {
   /**
    * APIProperty: extractWaypoints
    * {Boolean} Extract waypoints from GPX. (default: true)
    */
    extractWaypoints: true,
    
   /**
    * APIProperty: extractTracks
    * {Boolean} Extract tracks from GPX. (default: true)
    */
    extractTracks: true,
    
   /**
    * APIProperty: extractRoutes
    * {Boolean} Extract routes from GPX. (default: true)
    */
    extractRoutes: true,
    
    /**
     * APIProperty: extractAttributes
     * {Boolean} Extract feature attributes from GPX. (default: true)
     *     NOTE: Attributes as part of extensions to the GPX standard may not
     *     be extracted.
     */
    extractAttributes: true,
    
    /**
     * Constructor: OpenLayers.Format.GPX
     * Create a new parser for GPX.
     *
     * Parameters:
     * options - {Object} An optional object whose properties will be set on
     *     this instance.
     */
    initialize: function(options) {
        OpenLayers.Format.XML.prototype.initialize.apply(this, [options]);
    },
    
    /**
     * APIMethod: read
     * Return a list of features from a GPX doc
     *
     * Parameters:
     * doc - {Element} 
     *
     * Returns:
     * An Array of <OpenLayers.Feature.Vector>s
     */
    read: function(doc) {
        if (typeof doc == "string") { 
            doc = OpenLayers.Format.XML.prototype.read.apply(this, [doc]);
        }
        var features = [];
        
        if (this.extractWaypoints) {
            var waypoints = doc.getElementsByTagName("wpt");
            for (var l = 0; l < waypoints.length; l++) {
                var data = {
                  'type': 'wpt',
                  'title': this.getChildValue(waypoints[l], '*', 'name', Drupal.t('Untitled')),
                  'description': this.getChildValue(waypoints[l], '*', 'desc', Drupal.t('No description')),
                  'elevation': this.getChildValue(waypoints[l], '*', 'ele'),
                  'time': this.getChildValue(waypoints[l], '*', 'time')
                };
                var geometry = new OpenLayers.Geometry.Point(waypoints[l].getAttribute("lon"), waypoints[l].getAttribute("lat"));
                
                features.push(new OpenLayers.Feature.Vector(geometry, data));
            }
        }
        
        if (this.extractTracks) {
            var tracks = doc.getElementsByTagName("trk");
            for (var i = 0; i < tracks.length; i++) {
                // Attributes are only in trk nodes, not trkseg nodes
                var data = {
                  'type': 'trk',
                  'title': this.getChildValue(tracks[i], '*', 'name', Drupal.t('Untitled')),
                  'description': this.getChildValue(tracks[i], '*', 'desc', Drupal.t('No description'))
                }
                
                var segs = this.getElementsByTagNameNS(tracks[i], tracks[i].namespaceURI, "trkseg");
                for (var j = 0; j < segs.length; j++) {
                    var track = this.extractSegment(segs[j], data);
                    features.push(track);
                }
            }
        }
        
        if (this.extractRoutes) {
            var routes = doc.getElementsByTagName("rte");
            for (var k = 0; k < routes.length; k++) {
                var data = {
                  'type': 'rte'
                };
                var route = this.extractSegment(routes[k], data);
                features.push(route);
            }
        }
        
        // transform geometry?
        if (this.internalProjection && this.externalProjection) {
            for (var i = 0; i < features.length; i++) {
                features[i].geometry.transform(this.externalProjection, 
                               this.internalProjection);
            }
        }
        
        return features;
    },
    
    extractSegment: function(segment, data) {
        var points = this.getElementsByTagNameNS(segment, segment.namespaceURI, data.type + 'pt');
        var point_features = [];
        for (var i = 0; i < points.length; i++) {
            point_features.push(new OpenLayers.Geometry.Point(points[i].getAttribute("lon"), points[i].getAttribute("lat")));
        }
        return new OpenLayers.Feature.Vector(new OpenLayers.Geometry.LineString(point_features), data);
    },
    
    /**
     * Method: getChildValue
     *
     * Parameters:
     * node - {DOMElement}
     * nsuri - {String} Child node namespace uri ("*" for any).
     * name - {String} Child node name.
     * def - {String} Optional string default to return if no child found.
     *
     * Returns:
     * {String} The value of the first child with the given tag name.  Returns
     *     default value or empty string if none found.
     */
    getChildValue: function(node, nsuri, name, def) {
        var value;
        try {
            value = this.getElementsByTagNameNS(node, nsuri, name)[0].firstChild.nodeValue;
        } catch(e) {
            value = (def == undefined) ? "" : def;
        }
        return value;
    },
    
    CLASS_NAME: "MappingKit.Format.GPX" 
});

