// $Id: GPX.Layer.js,v 1.1.2.4 2008/10/10 13:10:23 openwereld Exp $

/**
 * @requires OpenLayers/Layer/Vector.js
 * @requires OpenLayers/Layer/Markers.js
 */

/**
 * Class: OpenLayers.Layer.GPX
 * 
 * Inherits from:
 *  - <OpenLayers.Layer.Vector>
 *  - <OpenLayers.Layer.Markers>
 */
MappingKit.Layer.GPX = OpenLayers.Class(
  OpenLayers.Layer.Vector, OpenLayers.Layer.Markers, {

    /**
     * APIProperty: isBaseLayer
     * {Boolean} GPX layer is not a base layer by default. 
     */
    isBaseLayer: false,
    
    /**
      * Property: loaded
      * {Boolean} Flag for whether the GML data has been loaded yet.
      */
    loaded: false,

    /** 
     * Property: location 
     * {String} store url of text file 
     */
    location: null,

    /**
     * APIProperty: formatOptions
     * {Object} Hash of options which should be passed to the format when it is
     * created. Must be passed in the constructor.
     */
    formatOptions: null, 

    /**
     * Property: vectorMode
     * {Boolean} Should be calculated automatically.
     */
    vectorMode: true, 
    
    /**
     * Constructor: OpenLayers.Layer.GPX
     *
     * Parameters:
     * name - {String} 
     * url - {String} 
     * options - {Object} Hashtable of extra options to tag onto the layer
     */
    initialize: function(name, url, options) {
        OpenLayers.Layer.Markers.prototype.initialize.apply(this, [name, options]);
        OpenLayers.Layer.Vector.prototype.initialize.apply(this, [name, options]);
        
        this.location = url;
    },
    
    /**
     * APIMethod: destroy
     * Destroy this layer
     */
    destroy: function() {
        OpenLayers.Layer.Vector.prototype.destroy.apply(this, arguments);
        OpenLayers.Layer.Markers.prototype.destroy.apply(this, arguments);
    },
    
    /**
     * Method: loadGPX
     * Start the load of the GPX data. Don't do this when we first add the layer,
     * since we may not be visible at any point, and it would therefore be a waste.
     */
    loadGPX: function() {
        if (!this.loaded) {
            this.events.triggerEvent("loadstart");
            //OpenLayers.loadURL(this.location, null, this, this.parseData);
            if (OpenLayers.Request) {
                // OL 2.7
                OpenLayers.Request.GET({
                    url: this.location,
                    success: this.parseData,
                    scope: this
                });
            }
            else {
                // OL 2.6
                OpenLayers.loadURL(this.location, null, this, this.parseData);
            }
            this.loaded = true;
        }
    },
    
    /**
     * Method: moveTo
     * If layer is visible and GPX has not been loaded, load GPX.
     * 
     * Parameters:
     * bounds - {Object} 
     * zoomChanged - {Object} 
     * dragging - {Object} 
     */
    moveTo: function(bounds, zoomChanged, dragging) {
        
        //OpenLayers.Layer.prototype.moveTo.apply(this, arguments);
        OpenLayers.Console.debug('moveTo zoomChanged='+zoomChanged+'; dragging='+dragging+'; this.drawn='+this.drawn);
        
        if (this.visibility && !this.loaded) {
            //this.events.triggerEvent("loadstart");
            this.loadGPX();
        }
        
        if (!dragging) {
            this.renderer.root.style.visibility = "hidden";
            
            // force a reflow on gecko based browsers to actually hide the svg
            if (navigator.userAgent.toLowerCase().indexOf("gecko") != -1) {
                this.div.scrollLeft = this.div.scrollLeft;
            }
            
            var move = {
                'dx' : parseInt(this.map.layerContainerDiv.style.left),
                'dy': parseInt(this.map.layerContainerDiv.style.top)
            };
            this.div.style.left = -move.dx + "px";
            this.div.style.top = -move.dy + "px";
            
            // reposition markers
	    for(i = 0; i < this.markers.length; i++) {
		marker = this.markers[i];
		lonlat = this.map.getLayerPxFromLonLat(marker.lonlat);
                if (marker.icon.calculateOffset) {
                    marker.icon.offset = marker.icon.calculateOffset(marker.icon.size);
                }
                var offsetPx = lonlat.offset(marker.icon.offset);
		marker.icon.imageDiv.style.left = (offsetPx.x + move.dx) + "px";
		marker.icon.imageDiv.style.top = (offsetPx.y + move.dy) + "px";
	    }
            
            var extent = this.map.getExtent();
            this.renderer.setExtent(extent);
            
            this.renderer.root.style.visibility = "visible";
        }
        
        if (zoomChanged || !this.drawn) {
            // vector part
            for (var i = 0; i < this.features.length; i++) {
                var feature = this.features[i];
                this.drawFeature(feature);
            }
            
            // marker part
            for (var i = 0; i < this.markers.length; i++) {
                this.drawMarker(this.markers[i]);
            }
            
            this.drawn = true;
        }
    },
    
    /** 
     * Method: setMap
     * The layer has been added to the map. 
     * 
     * If there is no renderer set, the layer can't be used. Remove it.
     * Otherwise, give the renderer a reference to the map and set its size.
     * 
     * Parameters:
     * map - {<OpenLayers.Map>} 
     */
    setMap: function(map) {
        OpenLayers.Layer.Vector.prototype.setMap.apply(this, arguments);
    },
    
    /**
     * Method: parseData
     * Parse the data returned from the Events call.
     *
     * Parameters:
     * ajaxRequest - {XMLHttpRequest} 
     */
    parseData: function(ajaxRequest) {
        OpenLayers.Console.debug('parseData');
        var doc = ajaxRequest.responseXML;
        if (!doc || !doc.documentElement) {
            doc = OpenLayers.Format.XML.prototype.read(ajaxRequest.responseText);
        }
        
        var options = {};
        
        OpenLayers.Util.extend(options, this.formatOptions);
        
        if (this.map && !this.projection.equals(this.map.getProjectionObject())) {
            options.externalProjection = this.projection;
            options.internalProjection = this.map.getProjectionObject();
        }
        
        var format = new MappingKit.Format.GPX(options);
        var features = format.read(doc);
        
        var segments = [];
        for (var i = 0; i < features.length; i++) {
            var feature = features[i];
            
            // we don't support features with no geometry in the GPX
            // layer at this time. 
            if (!feature.geometry) {
                continue;
            }
            
            var data = feature.data ? feature.data : {'title': Drupal.t('Unknown'), 'description': Drupal.t('Unknown')};
            if (data.type == 'wpt') {
                //var location = feature.geometry.getBounds().getCenterLonLat();
                var location = new OpenLayers.LonLat(feature.geometry.x, feature.geometry.y);
                //
                data.icon = this.options.icon == null ? OpenLayers.Marker.defaultIcon() : this.options.icon.clone();
                data.popupSize = this.popupSize ? 
                                 this.popupSize.clone() :
                                 new OpenLayers.Size(250, 120); // width, height
                
                var contentHTML = '<div class="olLayerGeoRSSClose">[x]</div>'; 
                contentHTML += '<div class="olLayerGeoRSSTitle">';
                contentHTML += data.title;
                contentHTML += '</div>';
                contentHTML += '<div style="" class="olLayerGeoRSSDescription">';
                contentHTML += data.description;
                contentHTML += '</div>';
                data['popupContentHTML'] = contentHTML;
                
                var feature = new OpenLayers.Feature(this, location, data);
                var marker = feature.createMarker();
                marker.events.register('click', feature, this.markerClick);
                this.addMarker(marker);
            }
            else {
                // tracks & routes
                segments.push(feature);
            }
        }
        // add tracks & routes
        this.addFeatures(segments);
        
        this.events.triggerEvent("loadend");
    },
    
    /**
     * Method: markerClick
     *
     * Parameters:
     * evt - {Event} 
     */
    markerClick: function(evt) {
        var sameMarkerClicked = (this == this.layer.selectedFeature);
        this.layer.selectedFeature = (!sameMarkerClicked) ? this : null;
        for(var i=0; i < this.layer.map.popups.length; i++) {
            this.layer.map.removePopup(this.layer.map.popups[i]);
        }
        if (!sameMarkerClicked) {
            var popup = this.createPopup();
            OpenLayers.Event.observe(popup.div, "click",
                OpenLayers.Function.bind(function() { 
                    for(var i=0; i < this.layer.map.popups.length; i++) { 
                        this.layer.map.removePopup(this.layer.map.popups[i]); 
                    }
                }, this)
            );
            this.layer.map.addPopup(popup); 
        }
        OpenLayers.Event.stop(evt);
    },

    CLASS_NAME: "MappingKit.Layer.GPX"
});

